==> DMT_core-1.4.0.tar.gz: DMT_core-1.4.0/test/test_interface_ngspice/compare_ADS_real_modelcard.py <==
""" test ngspice vs ADS using a real modelcard.
"""
import logging
import os.path
from DMT.core import SimCon, Plot, DutType, Sweep, specifiers, McParameter
from DMT.ngspice import DutNgspice
from DMT.ADS import DutAds
from DMT.hl2 import McHicum, Hl2Model, VA_FILES

import numpy as np

logging.basicConfig(
    level=logging.DEBUG,
    format="%(levelname)s - %(message)s",
    filename=os.path.join("logs", "test_ADS.log"),
    filemode="w",
)


def compare_ngspice_ads_real_mc():
    mc = McHicum.load(os.path.join("test", "test_interface_ngspice", "real_mcard.mcard"))

    ###### Disable branches
    ## General settings
    mc = mc.get_clean_modelcard()
    mc.set_values(
        {
            "flcomp": 2.4,
            "flnqs": 0,
            "flsh": 1,
            "alrth": 0,
            # 'rth'   : 3700,
            # 'kavl'  : 0,
            # 'qavl': 10,
            # 'ibcis':0,
            # 'ibcxs':0,
            # 'ibeis':1.3e-18,
            # 'ireis':0,
            # 'ibets':0,
            # 'ibeps':0,
            # 'ireps':0,
            # 'itss'  :0,
            # 'rbx'   :10,
            # 'tr'    :0,
            # 'tbhrec':0,
            # 'rcx'   :1,
        },
        force=True,
    )
    mc.va_file = VA_FILES["L2V2.4.0_internal"]

    rth_de = McParameter(name="rth_de", value=0)
    mc.add(rth_de)

    # mc = McHicum(
    #     load_model_from_path=os.path.join('test', 'test_interface_ngspice', 'dietmar_output.lib'),
    # )
    # mc.va_file = VA_FILES['L2V2.4.0_internal']
    # mc.set_values({
    #     'alrth':0,
    #     'rth':1200,
    # })

    dut_ads = DutAds(
        None, DutType.npn, mc, nodes="C,B,E,S,T", reference_node="E"
    )  # ADS with VA file

    # create a sweep with ONE operating point
    sweepdef = [
        {
            "var_name": specifiers.FREQUENCY,
            "sweep_order": 4,
            "sweep_type": "CON",
            "value_def": [10e9],
        },
        # {'var_name':specifiers.VOLTAGE + 'B',  'sweep_order':3, 'sweep_type':'CON' , 'value_def':[0.85]},
        # {'var_name':specifiers.VOLTAGE + 'B',  'sweep_order':3, 'sweep_type':'CON' , 'value_def':[-2.0]},
        # {'var_name':specifiers.VOLTAGE + 'B',  'sweep_order':3, 'sweep_type':'LIN' , 'value_def':[0.3, 1, 91]},
        {
            "var_name": specifiers.VOLTAGE + "B",
            "sweep_order": 3,
            "sweep_type": "CON",
            "value_def": [0.9],
        },
        {
            "var_name": specifiers.VOLTAGE + "C",
            "sweep_order": 2,
            "sweep_type": "CON",
            "value_def": [2],
        },
        {
            "var_name": specifiers.VOLTAGE + "E",
            "sweep_order": 1,
            "sweep_type": "CON",
            "value_def": [0],
        },
        # {'var_name':specifiers.VOLTAGE + 'B',  'sweep_order':3, 'sweep_type':'CON' , 'value_def':[0.9]},
        # {'var_name':specifiers.VOLTAGE + 'C',  'sweep_order':2, 'sweep_type':'CON' , 'value_def':[2.]},
        # {'var_name':specifiers.VOLTAGE + 'E',  'sweep_order':1, 'sweep_type':'CON' , 'value_def':[0]},
    ]
    outputdef = ["I_C", "I_B"]
    othervar = {"TEMP": 300}
    sweep = Sweep("gummel", sweepdef=sweepdef, outputdef=outputdef, othervar=othervar)

    # create a sweep with ONE operating point
    sweepdef = [
        {
            "var_name": specifiers.FREQUENCY,
            "sweep_order": 4,
            "sweep_type": "CON",
            "value_def": [10e9],
        },
        # {'var_name':specifiers.VOLTAGE + 'B',  'sweep_order':3, 'sweep_type':'CON' , 'value_def':[0.85]},
        # {'var_name':specifiers.VOLTAGE + 'B',  'sweep_order':3, 'sweep_type':'CON' , 'value_def':[-2.0]},
        {
            "var_name": specifiers.VOLTAGE + "B",
            "sweep_order": 3,
            "sweep_type": "LIN",
            "value_def": [0.3, 1, 91],
        },
        # {'var_name':specifiers.VOLTAGE + 'B',  'sweep_order':3, 'sweep_type':'CON' , 'value_def':[0.9]},
        {
            "var_name": specifiers.VOLTAGE + "C",
            "sweep_order": 2,
            "sweep_type": "CON",
            "value_def": [2],
        },
        {
            "var_name": specifiers.VOLTAGE + "E",
            "sweep_order": 1,
            "sweep_type": "CON",
            "value_def": [0],
        },
        # {'var_name':specifiers.VOLTAGE + 'B',  'sweep_order':3, 'sweep_type':'CON' , 'value_def':[0.9]},
        # {'var_name':specifiers.VOLTAGE + 'C',  'sweep_order':2, 'sweep_type':'CON' , 'value_def':[2.]},
        # {'var_name':specifiers.VOLTAGE + 'E',  'sweep_order':1, 'sweep_type':'CON' , 'value_def':[0]},
    ]
    outputdef = ["I_C", "I_B"]
    othervar = {"TEMP": 300}
    sweep = Sweep("gummel", sweepdef=sweepdef, outputdef=outputdef, othervar=othervar)

    # first simulate ADS
    sim_con = SimCon(n_core=1, t_max=30)
    sim_con.append_simulation(dut=dut_ads, sweep=sweep)
    sim_con.run_and_read(force=False)

    # read initial conditions
    data_ads = dut_ads.get_data(sweep=sweep)
    ve = 0
    vb = data_ads["VBE"].to_numpy()[0]
    vc = data_ads["VCE"].to_numpy()[0]
    vbi = data_ads["VBiE"].to_numpy()[0]
    vbp = data_ads["VBpE"].to_numpy()[0]
    vci = data_ads["VCiE"].to_numpy()[0]
    vei = data_ads["VEiE"].to_numpy()[0]
    vt = data_ads["DTSH"].to_numpy()[0]
    delta = 1
    initial_conditions = {
        "icVE": ve * delta,
        "icVEi": vei * delta,
        "icVC": vc * delta,
        "icVCi": vci * delta,
        "icVB": vb * delta,
        "icVBi": vbi * delta,
        "icVBp": vbp * delta,
        "icVt": vt * delta,
    }

    # get rth stuff from ads
    rth_t = data_ads["rth_t"].to_numpy()
    pterm = data_ads["pterm"].to_numpy()
    dT = rth_t * pterm

    assert np.isclose(vt, dT[0], rtol=1e-3)

    # simulate ngspice using initial condition from ADS
    dut_ngspice = DutNgspice(
        None,
        DutType.npn,
        mc,
        nodes="C,B,E,S,T",
        reference_node="E",
        copy_va_files=False,  # ngspice without VA file! -> use internal
        simulator_command="ngspice",
        initial_conditions=initial_conditions,
        simulator_options={"GMIN": 1e-18, "ABSTOL": 1e-15},
    )
    sim_con.append_simulation(dut=dut_ngspice, sweep=sweep)
    sim_con.run_and_read(force=True)

    data_ngspice = dut_ngspice.get_data(sweep=sweep)
    try:
        data_ngspice.ensure_specifier_column(
            specifiers.TRANSIT_FREQUENCY, ports=dut_ngspice.ac_ports
        )
        data_ads.ensure_specifier_column(specifiers.TRANSIT_FREQUENCY, ports=dut_ads.ac_ports)
    except StopIteration:
        pass

    model = Hl2Model(model_name="hl2", version="2.4.3")
    kwargs = mc.to_kwargs()

    # ngspice operating point
    vbiei = data_ngspice["VBIEI"].to_numpy()
    vbici = data_ngspice["VBICI"].to_numpy()
    vciei = data_ngspice["VCIEI"].to_numpy()
    temp = data_ngspice["TK"].to_numpy()
    # ngspice currents
    ibei_ngspice = data_ngspice["IBEI"].to_numpy()
    ibei_ads = data_ads["IBIEI"].to_numpy()
    it_ngspice = data_ngspice["IT"].to_numpy()
    it_ads = data_ads["IT_"].to_numpy()
    # some assertions since I am at this point clueeless ... ##########################################
    # assert np.allclose(
    #     model.model_ibei_temp(vbe=vbiei, t_dev=temp,**kwargs),
    #     data_ngspice['IBEI'].to_numpy(),
    #     rtol=1e-2
    # )
    try:
        assert np.allclose(
            model.model_ibci_temp(vbc=vbici, t_dev=temp, **kwargs),
            data_ngspice["IBCI"].to_numpy(),
            rtol=1e-3,
        )  # not exactly zero due to gmin
    except AssertionError:
        pass  # gmin makes this assertion fail for small current
    assert np.allclose(
        model.model_ick_temp(vce=vciei, t_dev=temp, **kwargs),
        data_ngspice["ICK"].to_numpy(),
        rtol=1e-3,
    )  # not exactly zero due to gmin
    assert np.allclose(
        model.model_cjci_temp(vbc=vbici, t_dev=temp, **kwargs),
        data_ngspice["CMUI"].to_numpy(),
        rtol=1e-3,
    )
    # assert np.allclose(
    #     model.model_cjei_temp(vbe=vbiei, t_dev=temp, **kwargs),
    #     data_ngspice['CPII'].to_numpy(),
    #     rtol=1e-3
    # ) # does not work because of CdE

    # using currents

    # assert np.allclose(
    #     ibei_ngspice,
    #     ibei_ads,
    #     rtol=1e-3
    # )
    # assert np.allclose(
    #     it_ngspice,
    #     it_ads,
    #     rtol=1e-3
    # )

    return data_ngspice, data_ads


if __name__ == "__main__":
    data_ngspice, data_ads = compare_ngspice_ads_real_mc()

    # define plots
    plt_gummel = Plot(
        r"$I(V_{\mathrm{BE}})$",
        x_specifier=specifiers.VOLTAGE + "B" + "E",
        y_specifier=specifiers.CURRENT + "C",
        y_log=True,
        style="markers_lines",
        legend_location="upper left",
    )
    plt_ibei = Plot(
        r"$IBEI(V_{\mathrm{BE}})$",
        x_specifier=specifiers.VOLTAGE + "B" + "E",
        y_specifier=specifiers.CURRENT + "B",
        y_log=True,
        style="markers_lines",
    )
    plt_ibci = Plot(
        r"$IBCI(V_{\mathrm{BE}})$",
        x_specifier=specifiers.VOLTAGE + "B" + "E",
        y_specifier=specifiers.CURRENT + "B",
        y_log=True,
        style="markers_lines",
    )
    plt_it = Plot(
        r"IT(VBE)",
        x_specifier=specifiers.VOLTAGE + "B" + "E",
        y_specifier=specifiers.CURRENT + "C",
        y_log=True,
        style="markers_lines",
    )
    plt_temp = Plot(
        r"$dT(VBE)$",
        x_specifier=specifiers.VOLTAGE + "B" + "E",
        y_specifier=specifiers.TEMPERATURE,
        style="markers_lines",
    )
    plt_ft = Plot(
        r"FT(VBE)",
        x_specifier=specifiers.CURRENT + "C",
        y_specifier=specifiers.TRANSIT_FREQUENCY,
        x_log=True,
        style="markers_lines",
    )
    plt_vbici = Plot(
        r"VBICI(VBE)",
        x_specifier=specifiers.VOLTAGE + "B",
        y_specifier=specifiers.VOLTAGE + "Bi" + "Ci",
        style="markers_lines",
    )
    plt_vbiei = Plot(
        r"VBIEI(VBE)",
        x_specifier=specifiers.VOLTAGE + "B",
        y_specifier=specifiers.VOLTAGE + "Bi" + "Ei",
        style="markers_lines",
    )
    # end define plots

    # fill plots
    # gummel
    plt_gummel.add_data_set(data_ngspice["V_B"], data_ngspice["I_C"], label="ngspice $I_C$")
    plt_gummel.add_data_set(data_ads["V_B"], data_ads["I_C"], label="ADS $I_C$")
    plt_gummel.add_data_set(
        data_ngspice["V_B"],
        data_ngspice["I_B"],  # IB from ngspice short is wrong
        # data_ngspice['VBBP']/mc['rbx'].value,
        label="ngspice $I_B$",
    )
    # plt_gummel.add_data_set(
    #     data_ngspice['V_B'],
    #     data_ngspice['IBEI'], #IB from ngspice short is wrong
    #     #data_ngspice['VBBP']/mc['rbx'].value,
    #     label='ngspice $IBEI$'
    # )
    plt_gummel.add_data_set(data_ads["V_B"], data_ads["I_B"], label="ADS $I_B$")

    # ibei
    plt_ibei.add_data_set(data_ngspice["V_B"], data_ngspice["IBEI"], label="ngspice ibei")
    plt_ibei.add_data_set(data_ads["V_B"], data_ads["IBIEI"], label="ADS ibei")
    # ibci
    plt_ibci.add_data_set(data_ngspice["V_B"], data_ngspice["IBCI"], label="ngspice ibci")
    plt_ibci.add_data_set(data_ads["V_B"], data_ads["IBICI"], label="ADS ibci")
    # it
    plt_it.add_data_set(data_ngspice["V_B"], data_ngspice["IT"], label="ngspice it")
    plt_it.add_data_set(data_ads["V_B"], data_ads["IT_"], label="ADS it")
    # temp
    plt_temp.add_data_set(data_ngspice["V_B"], data_ngspice["V_T"], label="ngspice vt")
    plt_temp.add_data_set(data_ads["V_B"], data_ads["DTSH"], label="ADS it")
    # ft
    try:
        plt_ft.add_data_set(
            data_ngspice["I_C"], data_ngspice[specifiers.TRANSIT_FREQUENCY], label="ngspice "
        )
        plt_ft.add_data_set(data_ads["I_C"], data_ads[specifiers.TRANSIT_FREQUENCY], label="ADS ")
    except KeyError:
        pass
    # vbici
    plt_vbici.add_data_set(data_ngspice["V_B"], data_ngspice["VBICI"], label="ngspice ")
    plt_vbici.add_data_set(data_ads["V_B"], data_ads["_V_BiE"] - data_ads["_V_CiE"], label="ADS ")
    # vbiei
    plt_vbiei.add_data_set(data_ngspice["V_B"], data_ngspice["VBIEI"], label="ngspice ")
    plt_vbiei.add_data_set(data_ads["V_B"], data_ads["_V_BiE"] - data_ads["_V_EiE"], label="ADS ")

    plt_vbiei.plot_py(show=False)
    plt_vbici.plot_py(show=False)
    plt_ft.plot_py(show=False)
    plt_temp.plot_py(show=False)
    plt_it.plot_py(show=False)
    plt_ibci.plot_py(show=False)
    plt_ibei.plot_py(show=False)
    plt_gummel.plot_py(show=True)

==> DMT_core-1.8.0rc1.tar.gz: DMT_core-1.8.0rc1/test/test_interface_ngspice/compare_ADS_real_modelcard.py <==
""" test ngspice vs ADS using a real modelcard.
"""
import logging
from pathlib import Path
from DMT.core import SimCon, Plot, DutType, Sweep, specifiers, McParameter
from DMT.ngspice import DutNgspice
from DMT.ADS import DutAds
from DMT.hl2 import McHicum, Hl2Model, VA_FILES

import numpy as np

folder_path = Path(__file__).resolve().parent
test_path = folder_path.parent

logging.basicConfig(
    level=logging.DEBUG,
    format="%(levelname)s - %(message)s",
    filename=folder_path.parent.parent / "logs" / "test_ADS_real.log",
    filemode="w",
)


def compare_ngspice_ads_real_mc():
    mc = McHicum.load(folder_path / "real_mcard.mcard")

    ###### Disable branches
    ## General settings
    mc = mc.get_clean_modelcard()
    mc.set_values(
        {
            "flcomp": 2.4,
            "flnqs": 0,
            "flsh": 1,
            "alrth": 0,
            # 'rth'   : 3700,
            # 'kavl'  : 0,
            # 'qavl': 10,
            # 'ibcis':0,
            # 'ibcxs':0,
            # 'ibeis':1.3e-18,
            # 'ireis':0,
            # 'ibets':0,
            # 'ibeps':0,
            # 'ireps':0,
            # 'itss'  :0,
            # 'rbx'   :10,
            # 'tr'    :0,
            # 'tbhrec':0,
            # 'rcx'   :1,
        },
        force=True,
    )
    mc.va_file = VA_FILES["L2V2.4.0_internal"]

    rth_de = McParameter(name="rth_de", value=0)
    mc.add(rth_de)

    # mc = McHicum(
    #     load_model_from_path=os.path.join('test', 'test_interface_ngspice', 'dietmar_output.lib'),
    # )
    # mc.va_file = VA_FILES['L2V2.4.0_internal']
    # mc.set_values({
    #     'alrth':0,
    #     'rth':1200,
    # })

    dut_ads = DutAds(
        None, DutType.npn, mc, nodes="C,B,E,S,T", reference_node="E"
    )  # ADS with VA file

    # create a sweep with ONE operating point
    sweepdef = [
        {
            "var_name": specifiers.FREQUENCY,
            "sweep_order": 4,
            "sweep_type": "CON",
            "value_def": [10e9],
        },
        # {'var_name':specifiers.VOLTAGE + 'B',  'sweep_order':3, 'sweep_type':'CON' , 'value_def':[0.85]},
        # {'var_name':specifiers.VOLTAGE + 'B',  'sweep_order':3, 'sweep_type':'CON' , 'value_def':[-2.0]},
        # {'var_name':specifiers.VOLTAGE + 'B',  'sweep_order':3, 'sweep_type':'LIN' , 'value_def':[0.3, 1, 91]},
        {
            "var_name": specifiers.VOLTAGE + "B",
            "sweep_order": 3,
            "sweep_type": "CON",
            "value_def": [0.9],
        },
        {
            "var_name": specifiers.VOLTAGE + "C",
            "sweep_order": 2,
            "sweep_type": "CON",
            "value_def": [2],
        },
        {
            "var_name": specifiers.VOLTAGE + "E",
            "sweep_order": 1,
            "sweep_type": "CON",
            "value_def": [0],
        },
        # {'var_name':specifiers.VOLTAGE + 'B',  'sweep_order':3, 'sweep_type':'CON' , 'value_def':[0.9]},
        # {'var_name':specifiers.VOLTAGE + 'C',  'sweep_order':2, 'sweep_type':'CON' , 'value_def':[2.]},
        # {'var_name':specifiers.VOLTAGE + 'E',  'sweep_order':1, 'sweep_type':'CON' , 'value_def':[0]},
    ]
    outputdef = ["I_C", "I_B"]
    othervar = {"TEMP": 300}
    sweep = Sweep("gummel", sweepdef=sweepdef, outputdef=outputdef, othervar=othervar)

    # create a sweep with ONE operating point
    sweepdef = [
        {
            "var_name": specifiers.FREQUENCY,
            "sweep_order": 4,
            "sweep_type": "CON",
            "value_def": [10e9],
        },
        # {'var_name':specifiers.VOLTAGE + 'B',  'sweep_order':3, 'sweep_type':'CON' , 'value_def':[0.85]},
        # {'var_name':specifiers.VOLTAGE + 'B',  'sweep_order':3, 'sweep_type':'CON' , 'value_def':[-2.0]},
        {
            "var_name": specifiers.VOLTAGE + "B",
            "sweep_order": 3,
            "sweep_type": "LIN",
            "value_def": [0.3, 1, 91],
        },
        # {'var_name':specifiers.VOLTAGE + 'B',  'sweep_order':3, 'sweep_type':'CON' , 'value_def':[0.9]},
        {
            "var_name": specifiers.VOLTAGE + "C",
            "sweep_order": 2,
            "sweep_type": "CON",
            "value_def": [2],
        },
        {
            "var_name": specifiers.VOLTAGE + "E",
            "sweep_order": 1,
            "sweep_type": "CON",
            "value_def": [0],
        },
        # {'var_name':specifiers.VOLTAGE + 'B',  'sweep_order':3, 'sweep_type':'CON' , 'value_def':[0.9]},
        # {'var_name':specifiers.VOLTAGE + 'C',  'sweep_order':2, 'sweep_type':'CON' , 'value_def':[2.]},
        # {'var_name':specifiers.VOLTAGE + 'E',  'sweep_order':1, 'sweep_type':'CON' , 'value_def':[0]},
    ]
    outputdef = ["I_C", "I_B"]
    othervar = {"TEMP": 300}
    sweep = Sweep("gummel", sweepdef=sweepdef, outputdef=outputdef, othervar=othervar)

    # first simulate ADS
    sim_con = SimCon(n_core=1, t_max=30)
    sim_con.append_simulation(dut=dut_ads, sweep=sweep)
    sim_con.run_and_read(force=False)

    # read initial conditions
    data_ads = dut_ads.get_data(sweep=sweep)
    ve = 0
    vb = data_ads["VBE"].to_numpy()[0]
    vc = data_ads["VCE"].to_numpy()[0]
    vbi = data_ads["VBiE"].to_numpy()[0]
    vbp = data_ads["VBpE"].to_numpy()[0]
    vci = data_ads["VCiE"].to_numpy()[0]
    vei = data_ads["VEiE"].to_numpy()[0]
    vt = data_ads["DTSH"].to_numpy()[0]
    delta = 1
    initial_conditions = {
        "icVE": ve * delta,
        "icVEi": vei * delta,
        "icVC": vc * delta,
        "icVCi": vci * delta,
        "icVB": vb * delta,
        "icVBi": vbi * delta,
        "icVBp": vbp * delta,
        "icVt": vt * delta,
    }

    # get rth stuff from ads
    rth_t = data_ads["rth_t"].to_numpy()
    pterm = data_ads["pterm"].to_numpy()
    dT = rth_t * pterm

    assert np.isclose(vt, dT[0], rtol=1e-3)

    # simulate ngspice using initial condition from ADS
    dut_ngspice = DutNgspice(
        None,
        DutType.npn,
        mc,
        nodes="C,B,E,S,T",
        reference_node="E",
        copy_va_files=False,  # ngspice without VA file! -> use internal
        simulator_command="ngspice",
        initial_conditions=initial_conditions,
        simulator_options={"GMIN": 1e-18, "ABSTOL": 1e-15},
    )
    sim_con.append_simulation(dut=dut_ngspice, sweep=sweep)
    sim_con.run_and_read(force=True)

    data_ngspice = dut_ngspice.get_data(sweep=sweep)
    try:
        data_ngspice.ensure_specifier_column(
            specifiers.TRANSIT_FREQUENCY, ports=dut_ngspice.ac_ports
        )
        data_ads.ensure_specifier_column(specifiers.TRANSIT_FREQUENCY, ports=dut_ads.ac_ports)
    except StopIteration:
        pass

    model = Hl2Model(model_name="hl2", version="2.4.3")
    kwargs = mc.to_kwargs()

    # ngspice operating point
    vbiei = data_ngspice["VBIEI"].to_numpy()
    vbici = data_ngspice["VBICI"].to_numpy()
    vciei = data_ngspice["VCIEI"].to_numpy()
    temp = data_ngspice["TK"].to_numpy()
    # ngspice currents
    ibei_ngspice = data_ngspice["IBEI"].to_numpy()
    ibei_ads = data_ads["IBIEI"].to_numpy()
    it_ngspice = data_ngspice["IT"].to_numpy()
    it_ads = data_ads["IT_"].to_numpy()
    # some assertions since I am at this point clueeless ... ##########################################
    # assert np.allclose(
    #     model.model_ibei_temp(vbe=vbiei, t_dev=temp,**kwargs),
    #     data_ngspice['IBEI'].to_numpy(),
    #     rtol=1e-2
    # )
    try:
        assert np.allclose(
            model.model_ibci_temp(vbc=vbici, t_dev=temp, **kwargs),
            data_ngspice["IBCI"].to_numpy(),
            rtol=1e-3,
        )  # not exactly zero due to gmin
    except AssertionError:
        pass  # gmin makes this assertion fail for small current
    assert np.allclose(
        model.model_ick_temp(vce=vciei, t_dev=temp, **kwargs),
        data_ngspice["ICK"].to_numpy(),
        rtol=1e-3,
    )  # not exactly zero due to gmin
    assert np.allclose(
        model.model_cjci_temp(vbc=vbici, t_dev=temp, **kwargs),
        data_ngspice["CMUI"].to_numpy(),
        rtol=1e-3,
    )
    # assert np.allclose(
    #     model.model_cjei_temp(vbe=vbiei, t_dev=temp, **kwargs),
    #     data_ngspice['CPII'].to_numpy(),
    #     rtol=1e-3
    # ) # does not work because of CdE

    # using currents

    # assert np.allclose(
    #     ibei_ngspice,
    #     ibei_ads,
    #     rtol=1e-3
    # )
    # assert np.allclose(
    #     it_ngspice,
    #     it_ads,
    #     rtol=1e-3
    # )

    return data_ngspice, data_ads


if __name__ == "__main__":
    data_ngspice, data_ads = compare_ngspice_ads_real_mc()

    # define plots
    plt_gummel = Plot(
        r"$I(V_{\mathrm{BE}})$",
        x_specifier=specifiers.VOLTAGE + "B" + "E",
        y_specifier=specifiers.CURRENT + "C",
        y_log=True,
        style="markers_lines",
        legend_location="upper left",
    )
    plt_ibei = Plot(
        r"$IBEI(V_{\mathrm{BE}})$",
        x_specifier=specifiers.VOLTAGE + "B" + "E",
        y_specifier=specifiers.CURRENT + "B",
        y_log=True,
        style="markers_lines",
    )
    plt_ibci = Plot(
        r"$IBCI(V_{\mathrm{BE}})$",
        x_specifier=specifiers.VOLTAGE + "B" + "E",
        y_specifier=specifiers.CURRENT + "B",
        y_log=True,
        style="markers_lines",
    )
    plt_it = Plot(
        r"IT(VBE)",
        x_specifier=specifiers.VOLTAGE + "B" + "E",
        y_specifier=specifiers.CURRENT + "C",
        y_log=True,
        style="markers_lines",
    )
    plt_temp = Plot(
        r"$dT(VBE)$",
        x_specifier=specifiers.VOLTAGE + "B" + "E",
        y_specifier=specifiers.TEMPERATURE,
        style="markers_lines",
    )
    plt_ft = Plot(
        r"FT(VBE)",
        x_specifier=specifiers.CURRENT + "C",
        y_specifier=specifiers.TRANSIT_FREQUENCY,
        x_log=True,
        style="markers_lines",
    )
    plt_vbici = Plot(
        r"VBICI(VBE)",
        x_specifier=specifiers.VOLTAGE + "B",
        y_specifier=specifiers.VOLTAGE + "Bi" + "Ci",
        style="markers_lines",
    )
    plt_vbiei = Plot(
        r"VBIEI(VBE)",
        x_specifier=specifiers.VOLTAGE + "B",
        y_specifier=specifiers.VOLTAGE + "Bi" + "Ei",
        style="markers_lines",
    )
    # end define plots

    # fill plots
    # gummel
    plt_gummel.add_data_set(data_ngspice["V_B"], data_ngspice["I_C"], label="ngspice $I_C$")
    plt_gummel.add_data_set(data_ads["V_B"], data_ads["I_C"], label="ADS $I_C$")
    plt_gummel.add_data_set(
        data_ngspice["V_B"],
        data_ngspice["I_B"],  # IB from ngspice short is wrong
        # data_ngspice['VBBP']/mc['rbx'].value,
        label="ngspice $I_B$",
    )
    # plt_gummel.add_data_set(
    #     data_ngspice['V_B'],
    #     data_ngspice['IBEI'], #IB from ngspice short is wrong
    #     #data_ngspice['VBBP']/mc['rbx'].value,
    #     label='ngspice $IBEI$'
    # )
    plt_gummel.add_data_set(data_ads["V_B"], data_ads["I_B"], label="ADS $I_B$")

    # ibei
    plt_ibei.add_data_set(data_ngspice["V_B"], data_ngspice["IBEI"], label="ngspice ibei")
    plt_ibei.add_data_set(data_ads["V_B"], data_ads["IBIEI"], label="ADS ibei")
    # ibci
    plt_ibci.add_data_set(data_ngspice["V_B"], data_ngspice["IBCI"], label="ngspice ibci")
    plt_ibci.add_data_set(data_ads["V_B"], data_ads["IBICI"], label="ADS ibci")
    # it
    plt_it.add_data_set(data_ngspice["V_B"], data_ngspice["IT"], label="ngspice it")
    plt_it.add_data_set(data_ads["V_B"], data_ads["IT_"], label="ADS it")
    # temp
    plt_temp.add_data_set(data_ngspice["V_B"], data_ngspice["V_T"], label="ngspice vt")
    plt_temp.add_data_set(data_ads["V_B"], data_ads["DTSH"], label="ADS it")
    # ft
    try:
        plt_ft.add_data_set(
            data_ngspice["I_C"], data_ngspice[specifiers.TRANSIT_FREQUENCY], label="ngspice "
        )
        plt_ft.add_data_set(data_ads["I_C"], data_ads[specifiers.TRANSIT_FREQUENCY], label="ADS ")
    except KeyError:
        pass
    # vbici
    plt_vbici.add_data_set(data_ngspice["V_B"], data_ngspice["VBICI"], label="ngspice ")
    plt_vbici.add_data_set(data_ads["V_B"], data_ads["_V_BiE"] - data_ads["_V_CiE"], label="ADS ")
    # vbiei
    plt_vbiei.add_data_set(data_ngspice["V_B"], data_ngspice["VBIEI"], label="ngspice ")
    plt_vbiei.add_data_set(data_ads["V_B"], data_ads["_V_BiE"] - data_ads["_V_EiE"], label="ADS ")

    plt_vbiei.plot_py(show=False)
    plt_vbici.plot_py(show=False)
    plt_ft.plot_py(show=False)
    plt_temp.plot_py(show=False)
    plt_it.plot_py(show=False)
    plt_ibci.plot_py(show=False)
    plt_ibei.plot_py(show=False)
    plt_gummel.plot_py(show=True)
